CREATE OR REPLACE PACKAGE BODY http AS

  -- Write data to a TCP/IP connection
  PROCEDURE write(c IN OUT NOCOPY utl_tcp.connection,value VARCHAR2 := NULL) IS
    b PLS_INTEGER; -- Number of bytes written
  BEGIN
    b := utl_tcp.write_line(c,value);
  END;

  -- Write an HTTP Header to a TCP/IP connection
  PROCEDURE write_header(c IN OUT NOCOPY utl_tcp.connection,
                         name VARCHAR2,value VARCHAR2) IS
  BEGIN
    write(c,name || ': ' || value);
  END;

  ----------------------------------------------------------------------
  -- Given an http-based URL, parse our the domain, port number, and
  -- path name of the request.
  ----------------------------------------------------------------------
  PROCEDURE setHostPortPath(url VARCHAR2,proxyServer VARCHAR2,
                            proxyPort NUMBER, host OUT VARCHAR2,
                            port IN OUT NUMBER, path OUT VARCHAR2)
  IS
    temp VARCHAR2(400);
    slash NUMBER;
    colon NUMBER;
  BEGIN
    IF proxyServer IS NOT NULL THEN
      ------------------------------------------------------------------
      -- If a proxy server is set, then tcp connection is to the
      -- proxy server and the path is the full url to the actual page
      ------------------------------------------------------------------
      host := proxyServer;
      port := proxyPort;
      path := url;
    ELSE
      ------------------------------------------------------------------
      -- If no proxy server is set, then tcp connection is directly to
      -- the host/port specified in the url.
      ------------------------------------------------------------------
      temp        := SUBSTR(url,INSTR(url,'http://')+7);
      slash       := INSTR(temp,'/');
      IF slash > 0 THEN
        host := SUBSTR(temp,1,slash-1);
        path := SUBSTR(temp,slash);
      ELSE
        host := temp;
        path := '/';
      END IF;
      colon := INSTR(host,':');
      IF colon > 0 THEN
        port := TO_NUMBER(SUBSTR(host,colon+1));
        host := SUBSTR(host,1,colon-1);
      END IF;
    END IF;
  END;

  ----------------------------------------------------------------------
  -- Read the response back from a TCP/IP connection, checking for
  -- an HTTP 200 OK success header and taking notice of the
  -- Content-Type: header that might be sent by the server.
  ----------------------------------------------------------------------
  PROCEDURE get_response_from(c IN OUT NOCOPY utl_tcp.connection,
                              resp OUT VARCHAR2,
                              content_type OUT VARCHAR2) IS
    line      VARCHAR2(32767);
    firstLine BOOLEAN := TRUE;
    header    BOOLEAN := TRUE;
    success   BOOLEAN := TRUE;
  BEGIN
    WHILE (success) LOOP
      line := utl_tcp.get_line(c);
      IF firstLine THEN
        IF line NOT LIKE '%HTTP%200%' THEN
          success := FALSE;
        END IF;
        firstLine := FALSE;
      ELSE
        IF HEADER THEN
          IF line = utl_tcp.CRLF THEN
            HEADER := FALSE;
          ELSE
            IF UPPER(line) LIKE 'CONTENT-TYPE:%' THEN
              content_type := RTRIM(RTRIM(RTRIM(LTRIM(SUBSTR(line,14))),CHR(10)),CHR(13));
            END IF;
          END IF;
        ELSE
          resp := resp || line;
        END IF;
      END IF;
    END LOOP;
  EXCEPTION
    WHEN utl_tcp.end_of_input THEN
      NULL; -- end of input
  END;

  ----------------------------------------------------------------------
  -- Perform an HTTP POST request to the specified URL with the
  -- specified 'doc' content in the request body. Return the response
  -- as a String.
  ----------------------------------------------------------------------
  PROCEDURE post(doc                   VARCHAR2,
                 content_type          VARCHAR2,
                 url                   VARCHAR2,
                 resp              OUT VARCHAR2,
                 resp_content_type OUT VARCHAR2,
                 proxyServer           VARCHAR2 := NULL,
                 proxyPort             NUMBER   := 80) IS
    port      NUMBER := 80;
    host      VARCHAR2(400);
    path      VARCHAR2(400);
    line      VARCHAR2(32767);
    firstLine BOOLEAN := TRUE;
    header    BOOLEAN := TRUE;
    success   BOOLEAN := TRUE;
    c         utl_tcp.connection;
  BEGIN
    setHostPortPath(url,proxyServer,proxyPort,host,port,path);
    c := utl_tcp.open_connection(host, port);
    write(c,'POST '||path||' HTTP/1.0');
    write_header(c,'Content-Type',content_type);
    write_header(c,'Content-Length',LENGTH(doc));
    write(c);
    write(c,doc);
    utl_tcp.flush(c);
    get_response_from(c,resp,resp_content_type);
    utl_tcp.close_connection(c);
  EXCEPTION
    WHEN OTHERS THEN utl_tcp.close_connection(c); RAISE;
  END;

  ----------------------------------------------------------------------
  -- Perform an HTTP GET request to the specified URL and return the
  -- response as a String.
  ----------------------------------------------------------------------
  PROCEDURE get(url                   VARCHAR2,
                resp              OUT VARCHAR2,
                resp_content_type OUT VARCHAR2,
                proxyServer           VARCHAR2 := NULL,
                proxyPort             NUMBER   := 80) IS
    port      NUMBER := 80;
    host      VARCHAR2(400);
    path      VARCHAR2(400);
    msg       VARCHAR2(32767);
    c         utl_tcp.connection;
  BEGIN
    setHostPortPath(url,proxyServer,proxyPort,host,port,path);
    c := utl_tcp.open_connection(host, port);
    write(c,'GET '||path||' HTTP/1.0');
    write(c);
    utl_tcp.flush(c);
    get_response_from(c,resp,resp_content_type);
    utl_tcp.close_connection(c);
  EXCEPTION
    WHEN OTHERS THEN utl_tcp.close_connection(c); RAISE;
  END;
END;
